<?php

/**
 * @file
 * Contains \Drupal\refreshless\EventSubscriber\HtmlResponseSubscriber.
 */

namespace Drupal\refreshless\EventSubscriber;

use Drupal\Core\Render\HtmlResponse;
use Drupal\refreshless\RefreshlessPageState;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Response subscriber to handle HTML responses.
 */
class HtmlResponseSubscriber implements EventSubscriberInterface {

  /**
   * Constructs a HtmlResponseSubscriber object.
   *
   * @param \Drupal\refreshless\RefreshlessPageState $refreshless_page_state
   */
  public function __construct(RefreshlessPageState $refreshless_page_state) {
    $this->refreshlessPageState = $refreshless_page_state;
  }

  /**
   * Initializes Refreshless page state.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   The event to process.
   */
  public function initializeRefreshlessPageState(FilterResponseEvent $event) {
    $response = $event->getResponse();
    if (!$response instanceof HtmlResponse) {
      return;
    }

    $response->addAttachments(['drupalSettings' => ['refreshlessPageState' => $this->refreshlessPageState->build($response->getCacheableMetadata())]]);
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Run just before \Drupal\Core\EventSubscriber\HtmlResponseSubscriber,
    // which invokes
    // \Drupal\Core\Render\HtmlResponseAttachmentsProcessor::processAttachments,
    // which is what renders the attached drupalSettings into the HTML response.
    $events[KernelEvents::RESPONSE][] = ['initializeRefreshlessPageState', 1];

    return $events;
  }

}
